declare module EVA.Blobs {

  export namespace Errors {
    export const enum ZipBlobs {
      /**
       * Duplicate path '{0:string}' is not allowed
       */
      DuplicatePaths = 'ZipBlobs:DuplicatePaths'
    }

  }


  /**
   * Creates a new, empty blob. If you wish to store data in there directly, use the `StoreBlob` service instead.
   * 
   * Empty blobs are placeholders; eventually, you'll want to push data in there. This service allows you to create that placeholder, and it is expected you then use the
   * multipart or raw binary `POST` call to `/blob/{Guid}` to push your data.
   * 
   * The combination of `Category` and `OriginalName` must be unique, as the `GetBlobInfoService` allows retrieval based on these values. Internally, a fresh and unique `Guid` is
   * generated and returned from this request. The response also contains the full `Url` by which this blob can be accessed (which will contain aforementioned `Guid`). Use this
   * path to POST your data, too!
   * 
   * - [x] The `MimeType` is required, a few suggestions would be `application/pdf`, `text/csv` or `application/json` - but you probably already know this.
   * - [x] The `ExpireDate` is the date at which the given blob should expire. Leave this empty to *suggest* to keep it forever. We will bag it and get rid of it when the time is right. :hocho:
   * 
   * When retrieving the blob, it will be returned to you using the `MimeType` as `ContentType` header and a proper `Expires` header.
   */
  export interface CreateBlob {
    Category?: string;
    OriginalName?: string;
    MimeType?: string;
    ExpireDate?: string;
    LocationType?: string;
    Uri?: string;
  }

  export interface CreateBlobResponse {
    /**
     * The ID of the newly created blob.
     * Entity type: Blob
     */
    Guid: string;
    /**
     * The URL where the newly created blob can be accessed.
     */
    Url: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Deletes specified blob, either by `BlobID` or `BlobUrl`.
   */
  export interface DeleteBlob {
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  /**
   * Get information about a blob, either by `Guid` or by `Category` and `OriginalName`.
   */
  export interface GetBlobInfo {
    Guid?: string;
    Category?: string;
    OriginalName?: string;
  }

  export interface GetBlobInfoResponse {
    Guid: string;
    MimeType: string;
    OriginalName: string;
    Category: string;
    ExpireDate?: string;
    LastModificationTimeUtc: string;
    /**
     * In bytes
     */
    Size: number;
    Url: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetProductImagePlaceholder {
  }

  export interface GetPlaceholderResponse {
    Guid: string;
    Url: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List blobs intended for EVA user documentation.
   * 
   * - When not specifying `PageConfig.Filter.Category`, we will default to `Documentation`
   * - When `PageConfig.Filter.Category` does not start with `Documentation`, we will prefix it with `Documentation`
   */
  export interface ListBlobsForDocumentation {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListBlobsForDocumentationFilter>;
  }

  export interface ListBlobsForDocumentationResponse {
    Result?: EVA.Core.PagedResult<ListBlobsForDocumentationResponse.DocumentationBlob>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListBlobsForDocumentationResponse {
    export interface DocumentationBlob {
      ID: string;
      OriginalName: string;
      MimeType: string;
      Category: string;
      ExpireDate?: string;
      LocationType: string;
      Url: string;
    }

  }

  /**
   * List blobs intended for stencils.
   */
  export interface ListBlobsForStencil {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListBlobsForStencilFilter>;
  }

  export interface ListBlobsForStencilResponse {
    Result?: EVA.Core.PagedResult<ListBlobsForStencilResponse.StencilBlob>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListBlobsForStencilResponse {
    export interface StencilBlob {
      ID: string;
      OriginalName: string;
      MimeType: string;
      Category: string;
      ExpireDate?: string;
      LocationType: string;
      Url: string;
    }

  }

  export interface SetProductImagePlaceholder {
    MimeType: string;
    Data: string;
  }

  export interface SetPlaceholderResponse {
    Guid: string;
    Url: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new blob and stores `Data` to it.
   * 
   * The combination of `Category` and `OriginalName` must be unique, as the `GetBlobInfoService` allows retrieval based on these values. Internally, a fresh and unique `Guid` is
   * generated and returned from this request. The response also contains the full `Url` by which this blob can be accessed (which will contain aforementioned `Guid`).
   * 
   * - [x] The `MimeType` is required, a few suggestions would be `application/pdf`, `text/csv` or `application/json` - but you probably already know this.
   * - [x] The `ExpireDate` is the date at which the given blob should expire. Leave this empty to *suggest* to keep it forever. We will bag it and get rid of it when the time is right. :hocho:
   * 
   * When retrieving the blob, it will be returned to you using the `MimeType` as `ContentType` header and a proper `Expires` header.
   * 
   * *NOTE*: leaving `OriginalName` empty means EVA treats this file as a temporary one and it'll be removed after a set period. Which can be manipulated through the setting `Blobs:DefaultLifeTime`
   * which is the default expire time for these kinds of blobs in minutes.
   */
  export interface StoreBlob {
    Category?: string;
    OriginalName?: string;
    MimeType: string;
    Data: string;
    ExpireDate?: string;
  }

  export interface StoreBlobResponse {
    Guid: string;
    Url: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns a list of blobs as a zip file.
   */
  export interface ZipBlobs {
    /**
     * Name of the generated zip file, will default to 'download.zip' if not specified.
     */
    ZipName?: string;
    /**
     * List of blobs to be included in the zip file.
     */
    Blobs: ZipBlobs.BlobToZip[];
  }

  export namespace ZipBlobs {
    export interface BlobToZip {
      /**
       * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
       * Entity type: Blob
       */
      BlobID?: string;
      /**
       * Can be used for both standard- and secured blobs - must match environment assets URL.
       */
      BlobUrl?: string;
      /**
       * Path of the file in the resulting zip, including filename - when not set, files are set to root using their original blob name or else a random UUID.
       */
      Path?: string;
    }

  }

  export interface ZipBlobsResponse {
    ResourceInfo: EVA.Core.ResourceInfo;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

}
