declare module EVA.Carrier.Paazl {

  export namespace Errors {
    export const enum Paazl {
      /**
       * Specified `Carrier` is not configured as Shipping Method
       */
      ShippingMethodNotConfigured = 'Paazl:ShippingMethodNotConfigured',
      /**
       * Missing `ZipCode` that is required to retrieve shipping options
       */
      MissingZipCode = 'Paazl:MissingZipCode',
      /**
       * Missing `CountryID` that is required to retrieve shipping options
       */
      MissingCountryID = 'Paazl:MissingCountryID',
      /**
       * Invalid shipping option
       */
      InvalidShippingOption = 'Paazl:InvalidShippingOption',
      /**
       * Please select one address
       */
      SelectAddress = 'Paazl:SelectAddress',
      /**
       * Invalid shipping option address
       */
      InvalidAddress = 'Paazl:InvalidAddress',
      /**
       * Please select one delivery date
       */
      SelectDeliveryDate = 'Paazl:SelectDeliveryDate',
      /**
       * Invalid shipping delivery date
       */
      InvalidDeliveryDate = 'Paazl:InvalidDeliveryDate'
    }

  }


  /**
   * Returns a list of the available shipping options configured in Paazl, taking an `OrderID`. The related order needs to have a Shipping Address
   * set. The parameter `ExtendedDeliveryDateDetails` toggles the `Period` value contents of the `DeliveryDateDetails` property in the response.
   * 
   * This data should be presented to the user, so that a choice can be made between the available options. When the option represents a
   * "pick-up point", the user should additionally be able to select the desired address for delivery, of which the available options are also
   * included in above return data.
   * 
   * Optionally, supply a `ZipCode` and `CountryID` to deviate from specified orders' Shipping Address (eg. to support dynamic pickup points). Note
   * that you'll have to supply the same inputs to `PaazlUpdateShippingOptionsForOrder` if you want to use this feature!
   * 
   * :watch:	Any `Day` mentioned is a date without time, and so it's timezone is irrelevant. Any `From` and `Till` time is in local time to the
   * receipient or pick-up point, and needs to be presented to the user and saved with the `PaazlUpdateShippingOptionsForOrder` without any timezone
   * conversion.
   */
  export interface PaazlGetShippingOptionsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * @default false
     */
    ExtendedDeliveryDateDetails?: boolean;
    ZipCode?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    ShowUnavailable?: boolean;
  }

  export interface PaazlGetShippingOptionsForOrderResponse {
    Result: PaazlShippingOption[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface PaazlShippingOption {
    Type: string;
    Carrier: string;
    Description: string;
    Price?: number;
    DisplayPrice?: number;
    PartOfDay: string;
    DayOfWeek: string;
    IsAvailable: boolean;
    DeliveryDateDetails: PaazlDeliveryDateDetail[];
    Addresses: PaazlShippingAddress[];
    ShippingMethod?: EVA.Core.ShippingMethodDto;
  }

  export interface PaazlDeliveryDateDetail {
    Day: string;
    Period: PaazlPeriod;
  }

  export interface PaazlPeriod {
    From: string;
    Till: string;
  }

  export interface PaazlShippingAddress {
    Name: string;
    Type: string;
    Code: string;
    StreetAndHouseNumber: string;
    ZipCode: string;
    City: string;
    CountryID: string;
    Latitude: number;
    Longitude: number;
    Distance?: number;
    Price?: number;
    DisplayPrice?: number;
    AdditionalInfo: string;
    OpeningHours: PaazlOpeningHours[];
  }

  export interface PaazlOpeningHours {
    DayOfWeek: number;
    AlwaysOpen: boolean;
    Period: PaazlPeriod;
  }

  /**
   * The user choice should be passed to the `PaazlUpdateShippingOptionsForOrder` service, using the same `OrderID` as before, and the selected
   * `PaazlShippingOption` as previously returned by the first service. When not using a pickup point service, leave the `Address` array empty when
   * returning it. When using a pickup point service, return the array with a single element.
   * 
   * If you called `PaazlGetShippingOptionsForOrder` with a `ZipCode`/`CountryID` override, be sure to specify the same values here!
   * 
   * This service is a wrapper around various other services;
   * 
   * * `SetShippingMethod` service to set the entire order to specified shipping method (property `Type` will be mapped to an EVA shipping method)
   * * `UpdateOrderAddresses` service to set the pickup point address as `ShippingAddress` (in case of a pickup point selection)
   * * `SetDeliveryOrderData` service to store the additional delivery data for the order
   * * And it will recalculate the shipping costs, based on this new selection
   */
  export interface PaazlUpdateShippingOptionsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    ShippingOption: PaazlShippingOption;
    ZipCode?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

}
