declare module EVA.Carrier.PostNL {

  export namespace Errors {
    export const enum PostNL {
      /**
       * There were no valid coordinates provided.
       */
      NoValidCoordinates = 'PostNL:NoValidCoordinates',
      /**
       * There is not ShippingAddress set on the given Order.
       */
      MissingShippingAddress = 'PostNL:MissingShippingAddress'
    }

  }


  /**
   * Creates a shipment barcode & label from PostNL, setting barcode as tracking number and attach label as order document, while also printing it to your current or optionally specified station.
   */
  export interface PostNLCreateShippingLabelForShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * List the available PostNL PickupPoints for the given order, in the requested area.
   */
  export interface PostNLGetAvailablePickUpLocationsForArea {
    /**
     * Entity type: Order
     */
    OrderID: number;
    NorthWest: PostNLGetAvailablePickUpLocationsForArea.Coordinate;
    SouthEast: PostNLGetAvailablePickUpLocationsForArea.Coordinate;
  }

  export namespace PostNLGetAvailablePickUpLocationsForArea {
    export interface Coordinate {
      Latitude: number;
      Longitude: number;
    }

  }

  export interface PostNLGetAvailablePickUpLocationsForAreaResponse {
    Locations: Location[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface Location {
    LocationCode: string;
    Name: string;
    Distance?: number;
    Latitude?: number;
    Longitude?: number;
    Address: LocationAddress;
    DeliveryOptions: string[];
    OpeningHours: LocationOpeningHours;
    PartnerName: string;
    PhoneNumber: string;
    RetailNetworkID: string;
    Saleschannel: string;
    TerminalType: string;
  }

  export interface LocationAddress {
    Countrycode: string;
    Zipcode: string;
    HouseNr: string;
    Street: string;
    City: string;
    Remark: string;
  }

  export interface LocationOpeningHours {
    Monday: string[];
    Tuesday: string[];
    Wednesday: string[];
    Thursday: string[];
    Friday: string[];
    Saturday: string[];
    Sunday: string[];
  }

  /**
   * List the available PostNL PickupPoints for the given order. The locations will be based on the ShippingAddress of the Order.
   */
  export interface PostNLGetAvailablePickUpLocationsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface PostNLGetAvailablePickUpLocationsForOrderResponse {
    Locations: Location[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the available PostNL timeframe options for the given order.
   * 
   * * The `StartDate` and `EndDate` parameters are required and inclusive, the time portions will be ignored.
   * * Will cross-reference PostNL timeframes with EVA stock availability and return only valid options for both
   * * When `SkipAvailabilityChecks` is true, the ExpectedAvailabilityDate on the lines is ignored
   * 
   * Only lines set to PostNL will be used for the EVA stock availability check; unless no lines apply, in which case all lines are considered.
   */
  export interface PostNLGetAvailableTimeframesForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    StartDate: string;
    EndDate: string;
    SkipAvailabilityChecks: boolean;
  }

  export interface PostNLGetAvailableTimeframesForOrderResponse {
    Frames: AvailableTimeframeDate[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AvailableTimeframeDate {
    Date: string;
    Timeframes: AvailableTimeframe[];
  }

  export interface AvailableTimeframe {
    Option: TimeframeOptions;
    StartDateTime: string;
    EndDateTime: string;
    Costs: number;
  }

  export const enum TimeframeOptions {
    /**
     * Daytime delivery
     */
    Daytime = 1,
    /**
     * Evening delivery
     */
    Evening = 2,
    /**
     * Morning delivery before 10:00
     */
    Morning = 4,
    /**
     * Morning delivery before 12:00
     */
    Noon = 8,
    /**
     * Sunday delivery
     */
    Sunday = 16,
    /**
     * Sameday delivery (must be used in combination with Evening)
     */
    Sameday = 32,
    /**
     * Afternoon delivery before 17:00
     */
    Afternoon = 64,
    All = 127
  }

  /**
   * Return the selected PostNL shipping options for the given Order.
   */
  export interface PostNLGetShippingOptionsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface PostNLGetShippingOptionsForOrderResponse {
    Date?: string;
    TimeframeOptions?: TimeframeOptions;
    PickUpLocationID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Set the requested PostNL shipping options for the given Order.
   */
  export interface PostNLUpdateShippingOptionsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Date?: string;
    TimeframeOptions?: TimeframeOptions;
    PickUpLocationID?: string;
  }

}
