declare module EVA.DataLake {

  export namespace Errors {
    export const enum DataLakeBackfill {
      /**
       * DataLake export is disabled.
       */
      DataLakeExportDisabled = 'DataLakeBackfill:DataLakeExportDisabled',
      /**
       * Provide either an id, a list of ids or a from/to range.
       */
      BatchInformationRequired = 'DataLakeBackfill:BatchInformationRequired',
      /**
       * Failed to start DataLake backfill process.
       */
      DataLakeBackfillProcessingFailed = 'DataLakeBackfill:DataLakeBackfillProcessingFailed',
      /**
       * You requested {requested:number} documents, but your max was set to {limit:number}.
       */
      RequestedQuantityExceedsLimit = 'DataLakeBackfill:RequestedQuantityExceedsLimit',
      /**
       * UserBatches backfill only allows from/to range.
       */
      UserBatchesBackfillOnlyAllowsFromTo = 'DataLakeBackfill:UserBatchesBackfillOnlyAllowsFromTo',
      /**
       * Backfill of type '{0:DataLakeBackfillType}' only accepts IDs.
       */
      BackfillTypeOnlyAcceptsIDs = 'DataLakeBackfill:BackfillTypeOnlyAcceptsIDs'
    }

  }


  /**
   * Returns a list of all available Data Lake models.
   */
  export interface GetAvailableDataLakeModels {
  }

  export interface GetAvailableDataLakeModelsResponse {
    Models: GetAvailableDataLakeModelsResponse.Model[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAvailableDataLakeModelsResponse {
    export interface Model {
      Name: string;
    }

  }

  /**
   * Define which DataLake models should be ignored when exporting to the configured DataLake.
   */
  export interface UpdateDataLakeConfiguration {
    IgnoredModels?: string[];
  }

  /**
   * Manually re-export specific DataLake documents.
   * 
   * These types are currently supported;
   * 
   * * `FinancialPeriods` = 1
   * * `Orders` = 2
   * * `Invoices` = 3
   * * `Users` = 4
   * * `Discounts` = 5
   * * `UserBatches` = 6 (can only be used with `From` and `To`)
   * * `Cases` = 7
   * * `Shipments` = 8
   * * `ZonedCycleCounts` = 9
   * * `UserTasks` = 10
   * * `StockMutations` = 11
   * * `UserLoyaltyPointMutations` = 12 (can only be used with the `ID` or `IDs`. These need to be the user ID's of which the loyalty point mutation will be exported.)
   * * `LoyaltyProgramGroups` = 13 (can only be used with the `ID` or `IDs`.)
   * * `LoyaltyPrograms` = 14 (can only be used with the `ID` or `IDs`.)
   * * `FullStockCounts` = 15 (can only be used with the `ID` or `IDs`.)
   * 
   * Specify either a single `ID`, a list of `IDs`, or a datetime range `From` and `To` (inclusive).
   * 
   * As mandatory safety feature, specify the `MaxNumberOfDocuments` you'd like to run this for. When the amount of documents through above
   * selection exceeds this number, this service will export NOTHING and return to you an error with the amount of documents it prevented you
   * from exporting, so you can adjust your selection or expand your max.
   * 
   * :warning: Triggering large amounts of data may impact system performance!
   * 
   * :point_right: The `ID` you are returned can be used in subsequent services `DataLakeBackfillStatus` and `DataLakeBackfillCancel`.
   */
  export interface DataLakeBackfill {
    Type: DataLakeBackfillType;
    MaxNumberOfDocuments: number;
    /**
     * Entity type: DataModel
     */
    ID?: number;
    /**
     * Entity type: DataModel
     */
    IDs?: number[];
    From?: string;
    To?: string;
  }

  export const enum DataLakeBackfillType {
    FinancialPeriods = 1,
    Orders = 2,
    Invoices = 3,
    Users = 4,
    Discounts = 5,
    UserBatches = 6,
    Cases = 7,
    Shipments = 8,
    ZonedCycleCounts = 9,
    UserTasks = 10,
    StockMutations = 11,
    UserLoyaltyPointMutations = 12,
    LoyaltyProgramGroups = 13,
    LoyaltyPrograms = 14,
    FullStockCounts = 15,
    StockNotifications = 16
  }

  export interface DataLakeBackfillResponse {
    ID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Retrieve the status from a previous `DataLakeBackfill` request.
   */
  export interface DataLakeBackfillStatus {
    ID: string;
  }

  export interface DataLakeBackfillStatusResponse {
    Total?: number;
    Done?: number;
    Progress?: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Cancel a previous `DataLakeBackfill` request.
   */
  export interface DataLakeBackfillCancel {
    ID: string;
  }

  /**
   * Preview a datalake export. Use this service sparingly. Not all models can be previewed. This is only for testing purposes. Only a single ID can be specified at a time
   */
  export interface DataLakePreview {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    /**
     * Entity type: StockMutation
     */
    StockMutationID?: number;
    /**
     * Entity type: Survey
     */
    SurveyID?: number;
  }

  export interface DataLakePreviewResponse {
    Order?: DataLakeOrder;
    User?: DataLakeUser;
    FinancialPeriod?: DataLakeFinancialPeriod;
    StockMutation?: DataLakeStockMutation;
    Survey?: DataLakeSurvey;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface DataLakeOrder {
    DataLakeType: string;
    GenerationTime: string;
    ID: string;
    UserAgent: string;
    BackendID: string;
    BackendSystemID: string;
    Properties: number;
    OrderProperties: string[];
    Description: string;
    Type: string;
    GroupID?: string;
    CreationTime: string;
    LastModificationTime?: string;
    CaptureDate?: string;
    CreatedByID: string;
    Remark: string;
    CurrencyID: string;
    IsCompleted: boolean;
    IsPaid: boolean;
    IsShipped: boolean;
    IsTaxExempt: boolean;
    HasReturns: boolean;
    PaymentDate?: string;
    ShipmentDate?: string;
    CompletionDate?: string;
    InvoiceDate?: string;
    PlacementDate?: string;
    BillingAddress: DataLakeAddress;
    ShippingAddress: DataLakeAddress;
    Customer: DataLakeSimpleUser;
    Lines: DataLakeOrderLine[];
    Payments: DataLakePayment[];
    TotalAmountIncludingTaxes: number;
    TotalAmountInvoiced: number;
    TotalItems: number;
    TotalLineCount: number;
    OriginatingOrganizationUnit: DataLakeSimpleOrganizationUnit;
    SoldToOrganizationUnit: DataLakeSimpleOrganizationUnit;
    SoldFromOrganizationUnit: DataLakeSimpleOrganizationUnit;
    ShipToOrganizationUnit: DataLakeSimpleOrganizationUnit;
    ShipFromOrganizationUnit: DataLakeSimpleOrganizationUnit;
    PickupOrganizationUnit: DataLakeSimpleOrganizationUnit;
    CustomStatus: string[];
    OrderLedgers: DataLakeOrderLedger[];
    CustomFields: DataLakeSimpleCustomField[];
    GiftWrapping: DataLakeGiftWrappingOptions;
    Amounts: EVA.Core.OpenOrderAmounts;
    OrderCustomStatus: DataLakeEnum.WithBackendID;
    OrderCustomType: DataLakeEnum.WithBackendID;
    LoyaltyProgramID: string;
    FiscalID: string;
    VatNumber: string;
    Status: string;
  }

  export interface DataLakeAddress {
    FirstName: string;
    LastName: string;
    AddressedTo: string;
    HouseNumber: string;
    Address1: string;
    Address2: string;
    ZipCode: string;
    Subdistrict: string;
    District: string;
    City: string;
    State: string;
    Region: string;
    CountryID: string;
    CountrySubdivisionID: string;
    Street: string;
  }

  export interface DataLakeSimpleUser {
    ID: string;
    BackendID: string;
    BackendSystemID: string;
    Nickname: string;
    EmailAddress: string;
    FirstName: string;
    LastName: string;
    FullName: string;
    Gender: string;
    DateOfBirth?: string;
    PlaceOfBirth: string;
    PhoneNumber: string;
    LanguageID: string;
    CountryID: string;
    Salutation: string;
    FiscalID: string;
    CustomFields: DataLakeSimpleCustomField[];
  }

  export interface DataLakeSimpleCustomField {
    CustomFieldID?: number;
    Name?: string;
    BackendID?: string;
    Value?: any;
  }

  export interface DataLakeOrderLine {
    ID: string;
    ParentID: string;
    ReturnedOrderID: string;
    ReturnedOrderLineID: string;
    Description: string;
    IsExported: boolean;
    CreationTime: string;
    LastModificationTime?: string;
    CreatedByID: string;
    UnitPrice: number;
    UnitPriceInTax: number;
    UnitPricePriceListID: string;
    UnitPriceBeforeCorrection?: number;
    UnitPriceBeforeCorrectionInTax?: number;
    UnitPriceCorrectionReason?: DataLakeEnum;
    OriginalUnitPrice?: number;
    OriginalUnitPriceInTax?: number;
    UnitCost?: number;
    OriginalAmountInTax?: number;
    TaxRate: number;
    CurrencyID: string;
    CostPriceCurrencyID: string;
    QuantityOrdered: number;
    QuantityCancelled: number;
    QuantityShipped: number;
    QuantityCommitted: number;
    QuantityExported: number;
    QuantityDelivered: number;
    QuantityReserved: number;
    QuantityInvoiced: number;
    TotalQuantityToShip: number;
    IsShipped: boolean;
    IsInvoiced: boolean;
    IsCancelled: boolean;
    IsCompleted: boolean;
    Type: string;
    Discounts: DataLakeDiscountDetail[];
    DiscountCouponID?: string;
    DiscountCouponCode?: string;
    Product: DataLakeProduct;
    SoldBy: DataLakeSimpleUser;
    SerialNumber?: string;
    IsCarryOut: boolean;
    IsDelivery: boolean;
    /**
     * Don't use this, rely on `IsCarryOut` and `IsDelivery` instead
     */
    IsReservation: boolean;
    /**
     * Don't use this, rely on `IsCarryOut` and `IsDelivery` instead
     */
    IsOrdered: boolean;
    RequestedDate?: string;
    AvailabilityStatus: string;
    ExpectedAvailabilityDate?: string;
    Remark: string;
    DiscountAmount: number;
    ShippingMethod: string;
    Reference: string;
    CustomOrderLineType: string;
    FulfillmentOrganizationUnitID: string;
    DiscountID: string;
    TaxCodeID: string;
    CustomFields: DataLakeSimpleCustomField[];
    ProductVariationLines?: DataLakeProductVariationLine[];
    ProductRequirements?: DataLakeProductRequirementValue[];
    Amounts: DataLakeOrderLineAmounts;
    ReturnReason: DataLakeReturnReason;
    TaxDetails?: DataLakeTaxDetails[];
  }

  export interface DataLakeEnum {
    ID: string;
    Name: string;
    Description: string;
  }

  export namespace DataLakeEnum {
    export interface WithBackendID {
      BackendID: string;
      ID: string;
      Name: string;
      Description: string;
    }

  }

  export interface DataLakeDiscountDetail {
    DiscountID: string;
    DiscountBackendID: string;
    DiscountOrderLineID: string;
    Amount: number;
    CompliancyRule: string;
  }

  export interface DataLakeProduct {
    ID: string;
    Name: string;
    BackendID: string;
    LedgerClassID: string;
    BackendSystemID: string;
    ManufacturerID: string;
  }

  export interface DataLakeProductVariationLine {
    ID: string;
    Label: string;
    Value: string;
  }

  export interface DataLakeProductRequirementValue {
    ID: string;
    Name: string;
    Value: any;
  }

  export interface DataLakeOrderLineAmounts {
    TotalAmountInTax: number;
    TotalTaxAmount: number;
    TotalAmount: number;
    NetTotalAmountInTax: number;
    NetTotalTaxAmount: number;
    NetTotalAmount: number;
    DiscountAmount: number;
  }

  export interface DataLakeReturnReason {
    ID: string;
    Name: string;
    BackendID: string;
  }

  export interface DataLakeTaxDetails {
    Name: string;
    Type: string;
    Rate: number;
    Amount: number;
  }

  export interface DataLakePayment {
    ID: string;
    BackendID: string;
    BackendSystemID: string;
    IsConfirmed: boolean;
    IsRefund: boolean;
    Code: string;
    Description: string;
    Amount: number;
    PaidAmount: number;
    CurrencyID: string;
    PaymentDate?: string;
    PaymentMethod: string;
    BackendRelationID: string;
    DeviceID?: string;
    CardID?: string;
    CardNumber?: string;
    CustomFields: DataLakeSimpleCustomField[];
  }

  export interface DataLakeSimpleOrganizationUnit {
    ID: string;
    Name: string;
    BackendID: string;
  }

  export interface DataLakeOrderLedger {
    ID: number;
    Type: string;
    Description: string;
    OldValue: string;
    OrderLineID: string;
    NewValue: string;
    CreationTime: string;
    CreatedBy: DataLakeSimpleUser;
  }

  export interface DataLakeGiftWrappingOptions {
    WrapOrder: boolean;
    WrapIndividually: boolean;
    Message: string;
    GreetingCardProductID: string;
    Wraps: DataLakeGiftWrappingOptionsWrap[];
  }

  export interface DataLakeGiftWrappingOptionsWrap {
    Lines: DataLakeGiftWrappingOptionsWrapLine[];
    GreetingCardProductID: string;
    Message: string;
  }

  export interface DataLakeGiftWrappingOptionsWrapLine {
    OrderLineID: string;
    Quantity: number;
  }

  export interface DataLakeUser {
    DataLakeType: string;
    ID: string;
    GenerationTime: string;
    CreationTime: string;
    LastModificationTime: string;
    CustomFields: DataLakeSimpleCustomField[];
    EmailAddress: string;
    BackendRelationID: string;
    Initials: string;
    FirstName: string;
    LastName: string;
    DateOfBirth?: string;
    Gender: string;
    PlaceOfBirth: string;
    LanguageID: string;
    CountryID: string;
    TypeID: number;
    Types: string[];
    ShippingAddress: DataLakeAddress;
    BillingAddress: DataLakeAddress;
    DeactivationDate?: string;
    LastSuccessfulLogin?: string;
    Nickname: string;
    CompanyID: string;
    FiscalID: string;
    BankAccount: string;
    PhoneNumber: string;
    TimeZone: string;
    BackendSystemID: string;
    BackendID: string;
    UserAgent: string;
    GlobalID: string;
    Title: string;
    Salutation: string;
    PreferredStoreID: string;
    OriginatingOrganizationUnitID: string;
    CustomID: string;
    PricingGroupID: string;
    UserSubscriptions: DataLakeUserSubscription[];
  }

  export interface DataLakeUserSubscription {
    SubscriptionID?: string;
    Status?: string;
    IsBlocked?: boolean;
    UserAgent?: string;
    SubscribeFromSourceOrganizationUnit?: DataLakeSimpleOrganizationUnit;
    SubscribeFromSourceUserAgent?: string;
    SubscribeFromSourceIdentifier?: string;
    UnsubscribeFromSourceOrganizationUnit?: DataLakeSimpleOrganizationUnit;
    UnsubscribeFromSourceUserAgent?: string;
    UnsubscribeFromSourceIdentifier?: string;
    UnsubscribeFromSourceReason?: string;
    OriginatingOrganizationUnit?: DataLakeSimpleOrganizationUnit;
    Subscription?: DataLakeSubscription;
  }

  export interface DataLakeSubscription {
    ID: string;
    LoyaltyProgramID: string;
    BackendID: string;
    Name: string;
  }

  export interface DataLakeFinancialPeriod {
    DataLakeType: string;
    ID: string;
    FinancialPeriodNumber: string;
    PreviousFinancialPeriodID: string;
    OpeningTime?: string;
    ClosingTime?: string;
    AutoOpened: boolean;
    AutoClosed: boolean;
    Status: string;
    OrganizationUnitID: string;
    /**
     * User who closed the period
     */
    User: DataLakeSimpleUser;
    /**
     * User who opened the period
     */
    OpenedByUser: DataLakeSimpleUser;
    CashDeposits: DataLakeCashDeposit[];
    CashJournals: DataLakeCashJournal[];
    Expenses: DataLakeCashExpense[];
    CashCorrections: DataLakeCashCorrection[];
    Employees: DataLakeFinancialPeriodEmployee[];
    LoyaltyProgram: DataLakeFinancialPeriodLoyaltyProgram;
    CurrencyID: string;
    CostPriceCurrencyID: string;
    FinancialPeriodAuditID: string;
    GenerationTime: string;
  }

  export interface DataLakeCashDeposit {
    DeviceID: string;
    CreationTime: string;
    CurrencyID: string;
    PaymentTypeID: string;
    PaymentType: DataLakePaymentType;
    Number: string;
    Amount: number;
    Status: string;
    CashTransactionLedgerID: string;
    CashTransactionLedger: DataLakeCashTransactionLedger;
    Details: DataLakeCashDepositDetail;
  }

  export interface DataLakePaymentType {
    PaymentMethodID: string;
    Name: string;
    Code: string;
    LedgerClassID: string;
    BackendRelationID: string;
    Category: string;
    CashJournalMethod: string;
  }

  export interface DataLakeCashTransactionLedger {
    ID: string;
    CurrencyID: string;
    CashJournalID: string;
    Description: string;
    Amount: number;
    CreationTime: string;
    CreatedByID: string;
    Details: DataLakeCashTransactionLedgerDetail;
    GroupID?: string;
    Type: string;
    PaymentTransactionID: string;
  }

  export interface DataLakeCashTransactionLedgerDetail {
    Coins: { [key: number]: number };
    BankNotes: { [key: number]: number };
    CashDeposits: { [key: number]: number };
  }

  export interface DataLakeCashDepositDetail {
    Coins: { [key: number]: number };
    BankNotes: { [key: number]: number };
  }

  export interface DataLakeCashJournal {
    ID: string;
    FinancialPeriodID: string;
    PreviousCashJournalID: string;
    PaymentTypeID: string;
    PaymentType: DataLakePaymentType;
    CurrencyID: string;
    Type: string;
    OpeningTime: string;
    ClosingTime?: string;
    OpeningAmount: number;
    ClosingAmount?: number;
    OpeningDeviation?: number;
    ClosingDeviation?: number;
    StationID: string;
    DeviceID: string;
    DeviceTypeID: string;
    DeviceName: string;
    OpeningDetailsObject: DataLakeCashJournalOpeningDetails;
    ClosingDetailsObject: DataLakeCashJournalClosingDetails;
  }

  export interface DataLakeCashJournalOpeningDetails {
    Coins: { [key: number]: number };
    BankNotes: { [key: number]: number };
    CashDeposits: { [key: number]: number };
  }

  export interface DataLakeCashJournalClosingDetails {
    Coins: { [key: number]: number };
    BankNotes: { [key: number]: number };
    CashDeposits: { [key: number]: number };
    Data: { [key: string]: any };
  }

  export interface DataLakeCashExpense {
    FinancialPeriodID: string;
    CashTransactionLedgerID: string;
    TypeID: string;
    TaxRate: number;
    Amount: number;
    TaxCodeID: string;
    Description: string;
    BlobID?: string;
    Type: DataLakeCashExpenseType;
  }

  export interface DataLakeCashExpenseType {
    ID: string;
    BackendID: string;
    Name: string;
  }

  export interface DataLakeCashCorrection {
    ID: string;
    FinancialPeriodID: string;
    Type: string;
    Amount: number;
    Remark: string;
    CashTransactionLedgerID: string;
    CashTransactionLedger: DataLakeCashTransactionLedger;
  }

  export interface DataLakeFinancialPeriodEmployee {
    UserID: string;
    CommissionShare?: number;
  }

  export interface DataLakeFinancialPeriodLoyaltyProgram {
    ID: string;
  }

  export interface DataLakeStockMutation {
    DataLakeType: string;
    GenerationTime: string;
    ID: string;
    Product: DataLakeProduct;
    OrganizationUnit: DataLakeSimpleOrganizationUnit;
    SourceStockLabel: string;
    DestinationStockLabel: string;
    Quantity: number;
    UnitCost: number;
    Reason: string;
    Remark: string;
    OrderLineID: string;
    BackendSystemID: string;
    BackendID: string;
    SerialNumber?: string;
    Ledgers: DataLakeProductLedger[];
    CreationTime: string;
    CreatedByID: string;
    AutomaticCorrectionQuantity: number;
  }

  export interface DataLakeProductLedger {
    StockLabel: string;
    Quantity: number;
  }

  export interface DataLakeSurvey {
    DataLakeType: string;
    GenerationTime: string;
    ID: string;
    Version: number;
    IsSystem: boolean;
    BackendID: string;
    OrganizationUnitSetID: string;
    Name: string;
    Description: string;
    IntroductionTitle: string;
    IntroductionMessage: string;
    IntroductionBlobID?: string;
    ClosingRemarkTitle: string;
    ClosingRemarkMessage: string;
    ClosingRemarkBlobID?: string;
    SurveyContext: DataLakeEnum;
    SurveyTrigger: string;
    SurveyDistributionHandler: string;
    StartDate: string;
    EndDate?: string;
    LastPublishedByID?: string;
    LastPublishedOn?: string;
    IsDeactivated: boolean;
    LastDeactivatedByID?: string;
    LastDeactivatedOn?: string;
    FirstQuestionID: string;
    Questions: DataLakeSurvey.DataLakeSurveyQuestion[];
  }

  export namespace DataLakeSurvey {
    export interface DataLakeSurveyQuestion {
      ID: string;
      Sequence: number;
      BackendID: string;
      Description: string;
      Remark: string;
      Text: string;
      DataType: DataLakeEnum;
      Required: boolean;
      DisplayType: string;
      InputHint: string;
      MinimumLength?: number;
      MaximumLength?: number;
      MinimumDate?: string;
      MaximumDate?: string;
      MinimumValue?: number;
      MaximumValue?: number;
      BlobID?: string;
      OptionalAnswers: { [key: string]: string };
      AllowMultipleAnswers: boolean;
      SurveyContextCustomFieldID?: string;
      StoreAnswerOnContextCustomField: boolean;
      UseContextCustomFieldDataAsPrefill: boolean;
    }

  }

}
