declare module EVA.GlobalBlue {

  export namespace Errors {
    export const enum GlobalBlue {
      /**
       * Order {0:number} has no valid EligibilityCheck yet.
       */
      NoValidEligibilityCheckDone = 'GlobalBlue:NoValidEligibilityCheckDone',
      /**
       * Order {0:number} is already issued with GlobalBlue.
       */
      OrderAlreadyIssued = 'GlobalBlue:OrderAlreadyIssued',
      /**
       * Order {0:number} is not invoiced yet, TaxFreeForm unavailable.
       */
      OrderNotInvoiced = 'GlobalBlue:OrderNotInvoiced',
      /**
       * Order {0:number} has no TaxFreeForm yet, request it first before printing.
       */
      NoTaxFreeFormAvailable = 'GlobalBlue:NoTaxFreeFormAvailable',
      /**
       * The supplier identifier is not valid, it should start with 308604.
       */
      InvalidGlobalBlueIdentifier = 'GlobalBlue:InvalidGlobalBlueIdentifier',
      /**
       * Global Blue is not enabled for the current organizationunit.
       */
      Disabled = 'GlobalBlue:Disabled'
    }

  }


  /**
   * Fetch the eligibility information for the current order.
   * 
   * When no BillingAddress is available on the Order, the country of residence has to be provided in the CountryCode field.
   * 
   * If the response TaxFreeAvailble is true, the user can proceed with calling the GlobalBlueStartTaxFree service.
   * 
   * If the response is false, tax free is not available. This can be due to several reasons:
   * - CountryEligibility == false: the selected CountryOfResidence is not eligible for TaxFree
   * - Amounts.AmountBelowMinimum: the total amount of the order is not enough to qualify for tax free, it should be at least Amounts.MinAmount
   * - Amounts.AmountOverMaximum: the total amount of the order is to much to qualify for tax free, it should be lower than Amounts.MaxAmount
   */
  export interface GlobalBlueGetEligibilityInformation {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Country of residence of the customer, overrides billingaddress CountryID
     * Entity type: Country
     */
    CountryCode?: string;
  }

  export interface GlobalBlueGetEligibilityInformationResponse {
    /**
     * Is this order eligible for a taxfree form
     */
    TaxFreeAvailable: boolean;
    /**
     * Is the provided country eligible for a taxfree form
     */
    CountryEligibility: boolean;
    Amounts: GlobalBlueGetEligibilityInformationResponse.GrossAmounts;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GlobalBlueGetEligibilityInformationResponse {
    export interface GrossAmounts {
      /**
       * Amount that should be added to the order to validate for TaxFree
       */
      AmountBelowMinimum: number;
      MinAmount: number;
      /**
       * Amount that should be removed from the order to validate for TaxFree
       */
      AmountOverMaximum: number;
      MaxAmount: number;
    }

  }

  /**
   * Fetch the reissue information for the current return order.
   * 
   * It will return the orders that will be voided, but are not fully returned, so it would make sense to reissue these orders.
   */
  export interface GlobalBlueGetOrdersToReissue {
    /**
     * The return order
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GlobalBlueGetOrdersToReissueResponse {
    Orders: GlobalBlueGetOrdersToReissueResponse.ReturnOrder[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GlobalBlueGetOrdersToReissueResponse {
    export interface ReturnOrder {
      /**
       * Entity type: Order
       */
      OrderID: number;
    }

  }

  /**
   * This service can be used to print the TaxFree form after it has been issued for an Order.
   * 
   * The OrderID is required, the StationID is optional when it's provided in the header.
   */
  export interface GlobalBluePrintForm {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * After a GlobalBlue card is scanned the number can be stored on the order to allow fast TaxFree handling.
   * 
   * To remove the GlobalBlueIdentifier from the Order call this service without providing the GlobalBlueIdentifier property.
   */
  export interface GlobalBlueSetGlobalBlueIdentifier {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    GlobalBlueIdentifier?: string;
  }

  /**
   * Start the GlobalBlue TaxFreeForm issuing process.
   * 
   * This will only work for Orders that passed the Eligibility check done by the GlobalBlueGetEligibilityInformation service.
   * The order should also be Invoiced.
   * 
   * The property PosIdentification should contain a unique identifier for the POS device performing this operation.
   */
  export interface GlobalBlueStartTaxFree {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Random GUID unique for the current POS/Companion instance
     */
    PosIdentification: string;
  }

  export interface GlobalBlueStartTaxFreeResponse {
    Url: string;
    FormData: { [key: string]: string };
    GroupID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

}
