declare module EVA.Payment.Adyen {

  export namespace Errors {
    export const enum Adyen {
      /**
       * Received Adyen error: {0:string}
       */
      Error = 'Adyen:Error',
      /**
       * Failed to create payment: {0:string}
       */
      CreatePaymentFailed = 'Adyen:CreatePaymentFailed',
      /**
       * Failed to create refund: {0:string}
       */
      RefundFailed = 'Adyen:RefundFailed',
      /**
       * Failed to create capture: {errorCode:string} / {errorMessage:string}
       */
      CaptureFailed = 'Adyen:CaptureFailed'
    }

    export const enum AdyenStoredValue {
      /**
       * A CardNumber is required to activate a giftcard
       */
      MissingCardNumber = 'AdyenStoredValue:MissingCardNumber',
      /**
       * Invalid gift card - card not found
       */
      GiftCardNotFound = 'AdyenStoredValue:GiftCardNotFound',
      /**
       * Invalid gift card - product not found
       */
      MissingProduct = 'AdyenStoredValue:MissingProduct',
      /**
       * Invalid gift card - missing PIN/SSC
       */
      MissingPinSsc = 'AdyenStoredValue:MissingPinSsc',
      /**
       * A CardNumber should have a length of {validLength:number} or {validLength2:number}, current length is {invalidLength:number}
       */
      InvalidCardNumberLength = 'AdyenStoredValue:InvalidCardNumberLength'
    }

  }


  /**
   * Validate the Adyen management api configuration.
   * 
   * Returns a list of messages that contains information about the configuration.
   */
  export interface AdyenValidateManagementConfiguration {
  }

  export interface AdyenValidateManagementConfigurationResponse {
    IsValid: boolean;
    Messages: AdyenValidateManagementConfigurationResponse.ValidationMessage[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace AdyenValidateManagementConfigurationResponse {
    export interface ValidationMessage {
      Error: boolean;
      Message: string;
    }

  }

  /**
   * Create a donation with the Adyen Giving program. This service can be called when the CreatePayment returned DonationAvailable.
   * 
   * The PaymentTransactionID and Amount are required. Amount is in the same currency as the PaymentTransaction.
   * 
   * Optionally a specific DonationAccount can be specified, this will fallback to the configured DonationAccount.
   */
  export interface AdyenDonate {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    Amount?: number;
    DonationAccount?: string;
  }

  export interface AdyenDonateResponse {
    Success: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Submits details for an Adyen payment created using `CreatePayment`. This step is only needed when no final state has been reached on the
   * `CreatePayment` request (for example for 3D Secure, or when getting redirected back directly from a payment method using an app switch).
   * 
   * The exact details, which need to be sent to this endpoint, are always specified in the `Properties.Data` parameter in the response of the
   * associated `CreatePayment` request.
   */
  export interface AdyenPaymentDetails {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID: number;
    Details: { [key: string]: string };
    PaymentData?: string;
    ThreeDSAuthenticationOnly?: boolean;
  }

  export interface AdyenPaymentDetailsResponse {
    RedirectUrl?: string;
    QrCode?: string;
    Data?: { [key: string]: any };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Retrieve all gateways supported by the current configuration of Adyen, matched with the existing payment types in EVA. `ID` is it's unique
   * identifier, `Name` is something that you can show the customer.
   * 
   * It is required to supply an `OrderID` or, the `CurrencyID` and `Amount`.
   * When no amount is given, the open amount of the order we be used.
   * It is recommended to specify the `Channel`:
   * - Web: 1
   * - iOS: 2
   * - Android: 3
   * 
   * Recommended way of calling this service is with only an `OrderID` and `Channel`.
   * 
   * * If present, the `Details` list will show a list of properties that this gateway supports. Every detail is identied with the `Key`, and to support
   * typed inputs, the `Type` is also given. If the `Type` is `select`, one of the items inside the `Items` list needs to be chosen. Here, again, the
   * `ID` is it's unique identifier, and the `Name` is something you can show the customer.
   * * If present, the `Brands` list will show a list of supported brands within this payment method, e.g. to list creditcards (mc, visa, amex, ...).
   * 
   * When creating the payment, use the `ID` of the gateway as `GatewayID`. Fill the payment properties `Details` with the properties specific to the
   * gateway, as indicated by this response's `Details` list.
   */
  export interface ListAdyenCheckoutGateways {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    Amount?: number;
    Channel?: AdyenChannel;
  }

  export const enum AdyenChannel {
    Web = 1,
    iOS = 2,
    Android = 3
  }

  export interface ListAdyenCheckoutGatewaysResponse {
    Gateways: ListAdyenCheckoutGatewaysResponse.Gateway[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListAdyenCheckoutGatewaysResponse {
    export interface Gateway {
      ID: string;
      /**
       * Entity type: PaymentType
       */
      PaymentTypeID: number;
      Name: string;
      /**
       * Requires at least v68 of the adyen integration
       */
      Issuers?: ListAdyenCheckoutGatewaysResponse.Item[];
      Brands: string[];
    }

    export interface Detail {
      ID: string;
      Type: string;
      Items: ListAdyenCheckoutGatewaysResponse.Item[];
    }

    export interface Item {
      ID: string;
      Name: string;
    }

  }

}
