declare module EVA.Privacy {

  export namespace Errors {
    export const enum PrivacyRemovalRequest {
      /**
       * Privacy removal already requested.
       */
      AlreadyRequested = 'PrivacyRemovalRequest:AlreadyRequested',
      /**
       * Cannot request privacy removal for this user type.
       */
      InvalidUserType = 'PrivacyRemovalRequest:InvalidUserType',
      /**
       * Can no longer request cancellation of this privacy removal request.
       */
      CannotBeCancelled = 'PrivacyRemovalRequest:CannotBeCancelled',
      /**
       * Only the user themselves can cancel the privacy removal request.
       */
      SelfCancelOnly = 'PrivacyRemovalRequest:SelfCancelOnly',
      /**
       * The maximum time the processing of a privacy removal request can be delayed is 24 hours.
       */
      InvalidDelay = 'PrivacyRemovalRequest:InvalidDelay'
    }

    export const enum RemovalRequestProcessor {
      /**
       * Some invoices are missing for removal request {0:number}, requesting generation.
       */
      InvoicesMissingForRemovalRequest = 'RemovalRequestProcessor:InvoicesMissingForRemovalRequest'
    }

  }


  export interface GetPrivacyDataRequestByID {
    /**
     * Entity type: DataRequest
     */
    ID: number;
  }

  export interface GetPrivacyDataRequestByIDResponse {
    /**
     * Entity type: DataRequest
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    UserFullName: string;
    Status: DataRequestStatus;
    GenerationTime?: string;
    /**
     * Entity type: Blob
     */
    BlobID?: string;
    Url: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export const enum DataRequestStatus {
    Requested = 0,
    Processing = 1,
    Ready = 2,
    Expired = 3
  }

  export interface CreatePrivacyDataRequest {
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  export interface CreatePrivacyDataRequestResponse {
    /**
     * Entity type: DataRequest
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface DeletePrivacyDataRequest {
    /**
     * Entity type: DataRequest
     */
    ID: number;
  }

  export interface ListPrivacyDataRequests {
    /**
     * Entity type: User
     */
    UserID?: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListPrivacyDataRequestsResponse {
    Result?: EVA.Core.PagedResult<ListPrivacyDataRequestsResponse.PrivacyDataRequest>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPrivacyDataRequestsResponse {
    export interface PrivacyDataRequest {
      /**
       * Entity type: DataRequest
       */
      ID: number;
      /**
       * Entity type: User
       */
      UserID: number;
      UserFullName: string;
      Status: DataRequestStatus;
      GenerationTime?: string;
      /**
       * Entity type: Blob
       */
      BlobID?: string;
      Url: string;
    }

  }

  export interface GetPrivacyOrderDataRemovals {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetPrivacyOrderDataRemovalsResponse {
    Results: GetPrivacyOrderDataRemovalsResponse.Model[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPrivacyOrderDataRemovalsResponse {
    export interface Model {
      /**
       * Entity type: Order
       */
      OrderID: number;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      Type: string;
    }

  }

  export interface CreatePrivacyOrderDataRemovalRequest {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    Type?: string;
  }

  /**
   * Get a privacy removal request by ID
   */
  export interface GetPrivacyRemovalRequestByID {
    /**
     * Entity type: RemovalRequest
     */
    ID: number;
  }

  export interface GetPrivacyRemovalRequestByIDResponse {
    /**
     * Entity type: RemovalRequest
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    UserFullName: string;
    Status: RemovalRequestStatus;
    RemovalTime?: string;
    Hash: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export const enum RemovalRequestStatus {
    Requested = 0,
    Processing = 1,
    Done = 2,
    Failed = 3,
    Cancelled = 4
  }

  /**
   * Create and schedule a privacy removal request.
   */
  export interface CreatePrivacyRemovalRequest {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * The delay before the removal request is processed. During this period the user themselves can cancel the request.
     */
    ProcessDelay?: string;
  }

  export interface CreatePrivacyRemovalRequestResponse {
    /**
     * Entity type: RemovalRequest
     */
    ID: number;
    Hash: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List privacy removal requests.
   */
  export interface ListPrivacyRemovalRequests {
    PageConfig?: EVA.Core.PageConfig<ListPrivacyRemovalRequestsFilter>;
  }

  export interface ListPrivacyRemovalRequestsFilter {
    /**
     * Entity type: User
     */
    UserIDs?: number[];
    /**
     * Entity type: RemovalRequest
     */
    IDs?: number[];
    Hashes?: string[];
    Status?: RemovalRequestStatus;
    CreationDateRange?: ListPrivacyRemovalRequestsFilter.DateRange;
  }

  export namespace ListPrivacyRemovalRequestsFilter {
    export interface DateRange {
      StartDate?: string;
      EndDate?: string;
    }

  }

  export interface ListPrivacyRemovalRequestsResponse {
    Result?: EVA.Core.PagedResult<ListPrivacyRemovalRequestsResponse.PrivacyRemovalRequest>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPrivacyRemovalRequestsResponse {
    export interface PrivacyRemovalRequest {
      /**
       * Entity type: RemovalRequest
       */
      ID: number;
      /**
       * Entity type: User
       */
      UserID: number;
      UserFullName: string;
      Status: RemovalRequestStatus;
      RemovalTime?: string;
      Hash: string;
    }

  }

  /**
   * Service where failed privacy removal requests can be reschuled for processing.
   */
  export interface ScheduleRemovalRequests {
    /**
     * Entity type: RemovalRequest
     */
    IDs?: number[];
  }

  /**
   * Async service to re-process a failed privacy removal request.
   */
  export interface ExecuteRemovalRequest {
    /**
     * Entity type: RemovalRequest
     */
    ID: number;
  }

  /**
   * Cancels a pending removal request. Can only be called by the user of the removal request themselves.
   */
  export interface CancelRemovalRequest {
    /**
     * Entity type: RemovalRequest
     */
    ID: number;
  }

}
