declare module EVA.Replenishment {

  export namespace Errors {
    export const enum ReplenishmentProposals {
      /**
       * This proposal is already running and cannot be retried at this moment.
       */
      AlreadyRunning = 'ReplenishmentProposals:AlreadyRunning'
    }

    export const enum DownloadReplenishmentProductsExcel {
      /**
       * You can't download more than 1000000 records through this service.
       */
      TooManyRecords = 'DownloadReplenishmentProductsExcel:TooManyRecords',
      /**
       * Replenishment product with ID {0:string} does not exist.
       */
      UnknownReplenishmentProduct = 'DownloadReplenishmentProductsExcel:UnknownReplenishmentProduct',
      /**
       * Missing product identifier for row {0:number}.
       */
      MissingProductIdentifier = 'DownloadReplenishmentProductsExcel:MissingProductIdentifier'
    }

    export const enum ReplenishmentProducts {
      /**
       * There was a duplicate record for OrganizationUnit/Product/StockLabel {0:string}/{1:string}/{2:string} within one time frame.
       */
      DuplicateRecord = 'ReplenishmentProducts:DuplicateRecord',
      /**
       * The MaximumQuantity {0:string} must be greater than or equal to the MinimumQuantity {1:string}.
       */
      InvalidQuantityRange = 'ReplenishmentProducts:InvalidQuantityRange'
    }

  }


  /**
   * Creates a new `ReplenishmentProposal`.
   * 
   * The new proposal is picked up and processed asynchronously and could take a while after creation to complete generating its results, depending on the
   * amount of sources, targets, products and the algorithm used. Check the `Status` property in the `GetReplenishmentProposal` response to see if it's
   * completed.
   * 
   * To allow selecting the output types for the generated proposal, the property `OutputTypes` can be set. It is a flags enum of the type
   * `ReplenishmentOutputTypes`. If the `OutputTypes` is not set, it will fall back to the setting `Replenishment:Outputs`.
   */
  export interface CreateReplenishmentProposal {
    /**
     * Required when `SourceOrganizationUnitSetID` is omitted
     * Entity type: OrganizationUnit
     */
    SourceOrganizationUnitIDs?: number[];
    /**
     * Required when `SourceOrganizationUnitIDs` is omitted
     * Entity type: OrganizationUnitSet
     */
    SourceOrganizationUnitSetID?: number;
    /**
     * Required when `TargetOrganizationUnitSetID` is omitted
     * Entity type: OrganizationUnit
     */
    TargetOrganizationUnitIDs?: number[];
    /**
     * Required when `TargetOrganizationUnitIDs` is omitted
     * Entity type: OrganizationUnitSet
     */
    TargetOrganizationUnitSetID?: number;
    /**
     * Optional list of stock labels to generate replenishment proposal for
     */
    StockLabels?: CreateReplenishmentProposal.StockLabelDto[];
    /**
     * Optional parameter to override the standard output types for created replenishment proposals
     */
    OutputTypes?: ReplenishmentOutputTypes;
  }

  export namespace CreateReplenishmentProposal {
    export interface StockLabelDto {
      /**
       * Ordering of the labels, to determine priority in scarcity
       */
      SequenceID: number;
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
    }

  }

  export const enum ReplenishmentOutputTypes {
    None = 0,
    AutoOrder = 1,
    AutoOrderAutoConfirm = 2,
    Debug = 4,
    Info = 8,
    Order = 16
  }

  export interface CreateReplenishmentProposalResponse {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get a `ReplenishmentProposal`.
   * 
   * * To get the possible sources of this proposal, use the 'GetReplenishmentProposalSources`.
   * * To get the targets of this proposal, use the 'GetReplenishmentProposalTargets`.
   * * To get the stock labels of this proposal, use the 'GetReplenishmentProposalStockLabels`.
   * * To get the results of this proposal, use the 'GetReplenishmentProposalResults`.
   */
  export interface GetReplenishmentProposal {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
  }

  export interface GetReplenishmentProposalResponse {
    Result: ReplenishmentProposalDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ReplenishmentProposalDto {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
    Status: ReplenishmentProposalStatus;
    Error: string;
    CreationTime: string;
    Blobs: ReplenishmentProposalBlobDto[];
  }

  export const enum ReplenishmentProposalStatus {
    New = 0,
    Processing = 1,
    Processed = 2,
    Error = 9
  }

  export interface ReplenishmentProposalBlobDto {
    TypeID: string;
    /**
     * Entity type: Blob
     */
    BlobID: string;
    Url: string;
  }

  /**
   * Get all Sources for a `ReplenishmentProposal`, identified by its `ID`. Supports filter option for `OrganizationUnitIDs`.
   */
  export interface GetReplenishmentProposalSources {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
  }

  export interface GetReplenishmentProposalSourcesResponse {
    Result: EVA.Core.OrganizationUnitDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get all Stock Labels for a `ReplenishmentProposal`, identified by its `ID`.
   */
  export interface GetReplenishmentProposalStockLabels {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
  }

  export interface GetReplenishmentProposalStockLabelsResponse {
    Result: StockLabelDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface StockLabelDto {
    SequenceID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
  }

  /**
   * Get all Targets for a `ReplenishmentProposal`, identified by its `ID`. Supports filter option for `OrganizationUnitIDs`.
   */
  export interface GetReplenishmentProposalTargets {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
  }

  export interface GetReplenishmentProposalTargetsResponse {
    Result: EVA.Core.OrganizationUnitDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Lists all Results for a `ReplenishmentProposal`, identified by its `ID`.
   * 
   * Through the `PageConfig`'s `Filter` parameter, you have the following filter options available:
   * 
   * * `SourceIDs` and/or `TargetIDs` as `OrganizationUnit`.`ID`'s
   * * `ProductIDs` as `Product`.`ID`s
   * * `CustomID` as `Product`.`CustomID`
   * * `BrandName` as `Product`.`Brand`.`Name`
   */
  export interface ListReplenishmentProposalResults {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
    PageConfig?: EVA.Core.PageConfig<ListReplenishmentProposalResultFilters>;
  }

  export interface ListReplenishmentProposalResultFilters {
    /**
     * Entity type: OrganizationUnit
     */
    SourceIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    TargetIDs: number[];
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    CustomID?: string;
    BrandName?: string;
  }

  export interface ListReplenishmentProposalResultsResponse {
    Result?: EVA.Core.PagedResult<ReplenishmentProposalResultListDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ReplenishmentProposalResultListDto {
    /**
     * Entity type: ReplenishmentProposalResult
     */
    ID: number;
    /**
     * Entity type: ReplenishmentProposal
     */
    ReplenishmentProposalID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SourceOrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    TargetOrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    Quantity: number;
    SourceOrganizationUnit: EVA.Core.OrganizationUnitDto;
    TargetOrganizationUnit: EVA.Core.OrganizationUnitDto;
    Product: EVA.Core.ProductDto;
    ProductName: string;
    UnitCost?: number;
    TotalCost?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
  }

  /**
   * List all `ReplenishmentProposal`'s.
   * 
   * Through the `PageConfig`'s `Filter` parameter, you have the following filter options available:
   * 
   * * `ID` for that specific `ReplenishmentProposal` you want
   * * `SourceIDs` and/or `TargetIDs` as `OrganizationUnit`.`ID`'s
   * * `CreatedAfter` and/or `CreatedBefore`
   * * `CreatedByName` (:underage: expensive filter, supports both `FirstName` and `LastName` filtering)
   */
  export interface ListReplenishmentProposals {
    PageConfig?: EVA.Core.PageConfig<ListReplenishmentProposalFilters>;
  }

  export interface ListReplenishmentProposalFilters {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    SourceIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    TargetIDs: number[];
    CreatedAfter?: string;
    CreatedBefore?: string;
    CreatedByName?: string;
  }

  export interface ListReplenishmentProposalsResponse {
    Result?: EVA.Core.PagedResult<ReplenishmentProposalListDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ReplenishmentProposalListDto {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
    StatusID: number;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreationTime: string;
    CreatedBy: EVA.Core.UserDto;
  }

  /**
   * Downloads an Excel for `UploadReplenishmentProductsExcel` service, filled with all currently existing records.
   * 
   * Specify either OrganizationUnitIDs or OrganizationUnitSetID. OrganizationUnitSetID takes precedence over OrganizationUnitIDs.
   * 
   * You can't download information for more than 1.000.000 product/organizationunit combinations.
   */
  export interface DownloadReplenishmentProductsExcel {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * @default false
     */
    ShowInactive?: boolean;
    /**
     * Only show products that match this filter.
     */
    Filters?: { [key: string]: EVA.Core.FilterModel };
    /**
     * Will add empty rows for product/organizationunit combinations that do not exist yet.
     */
    AddPlaceHolderRows?: boolean;
  }

  /**
   * Uploads an Excel retrieved from `DownloadReplenishmentProductsExcel`.
   * 
   * Records with `ID` will be modified, records without `ID` will be added. Omitted records will not be modified.
   * 
   * If you add a record that already exists, but is not in the excel due to some filtering (ou or products), it will also be updated.
   */
  export interface UploadReplenishmentProductsExcel {
    Data: string;
  }

  /**
   * Creates a new ReplenishmentProduct.
   */
  export interface CreateReplenishmentProduct {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    MinimumQuantity: number;
    MaximumQuantity: number;
    /**
     * Date, no time. Inclusive.
     */
    StartDate?: string;
    /**
     * Date, no time. Inclusive.
     */
    EndDate?: string;
  }

  export interface CreateReplenishmentProductResponse {
    /**
     * Entity type: ReplenishmentProduct
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Edits an existing ReplenishmentProduct.
   */
  export interface EditReplenishmentProduct {
    /**
     * Entity type: ReplenishmentProduct
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    MinimumQuantity: number;
    MaximumQuantity: number;
    /**
     * Date, no time. Inclusive.
     */
    StartDate?: string;
    /**
     * Date, no time. Inclusive.
     */
    EndDate?: string;
  }

  /**
   * Deletes an existing ReplenishmentProduct.
   */
  export interface DeleteReplenishmentProduct {
    /**
     * Entity type: ReplenishmentProduct
     */
    ID: number;
  }

  /**
   * List the ReplenishmentProducts, optionally with a `PageConfig` with a `Filter`.
   */
  export interface ListReplenishmentProducts {
    PageConfig?: EVA.Core.PageConfig<ListReplenishmentProductsFilters>;
  }

  export interface ListReplenishmentProductsFilters {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * @default false
     */
    ShowInactive?: boolean;
  }

  export interface ListReplenishmentProductsResponse {
    /**
     * Additional properties that are returned with each product. It is guaranteed these are present for all products, but it might be an empty string.
     */
    AdditionalProperties: string[];
    Result?: EVA.Core.PagedResult<ListReplenishmentProductsResponse.ReplenishmentProduct>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListReplenishmentProductsResponse {
    export interface ReplenishmentProduct {
      /**
       * Entity type: ReplenishmentProduct
       */
      ID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitBackendID?: string;
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductBackendID?: string;
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      MinimumQuantity: number;
      MaximumQuantity: number;
      /**
       * Date, no time. Inclusive.
       */
      StartDate?: string;
      /**
       * Date, no time. Inclusive.
       */
      EndDate?: string;
      /**
       * Additional properties for the product
       */
      AdditionalProperties: { [key: string]: string };
    }

  }

  /**
   * Imports a simple set of ReplenishmentProducts, creating new records and updating existing records.
   * 
   * Does not support `StartDate` and `EndDate`, and as such will error when existing records have these set.
   */
  export interface ImportReplenishmentProducts {
    Products: ImportReplenishmentProducts.ReplenishmentProduct[];
  }

  export namespace ImportReplenishmentProducts {
    export interface ReplenishmentProduct {
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      /**
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      MinimumQuantity: number;
      MaximumQuantity: number;
    }

  }

  /**
   * Retry generating a new `ReplenishmentProposal`.
   */
  export interface RetryReplenishmentProposal {
    /**
     * Entity type: ReplenishmentProposal
     */
    ID: number;
    /**
     * Optional parameter to override the standard output types for created replenishment proposals
     */
    OutputTypes?: ReplenishmentOutputTypes;
  }

}
