declare module EVA.Symphony {

  export namespace Errors {
    export const enum Symphony {
      /**
       * The sheet could not be saved due to compile errors: {0:string}
       */
      CannotSaveSheetDueToCompileErrors = 'Symphony:CannotSaveSheetDueToCompileErrors',
      /**
       * Sheet {id:number} cannot be compiled, errors: {errors:string}
       */
      SheetNotCompiling = 'Symphony:SheetNotCompiling'
    }

  }


  export interface BuildSheet {
    AST: { [key: string]: any };
    Sheet: string;
  }

  export interface BuildSheetResult {
    CodeSnippet: string;
    Sheet: string;
    HasErrors: boolean;
    Errors: ParseSheetError[];
    CompletionSuggestions: CompletionSuggestion[];
    AST: { [key: string]: any };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ParseSheetError {
    Type: EVA.Core.ParsingErrorType;
    Message: string;
    SourceLine: string;
    SourceLineNumber: number;
    SourceColumn: number;
  }

  export interface CompletionSuggestion {
    Label: string;
    Code: string;
    Type: CompletionSuggestionType;
    Documentation: string;
  }

  export const enum CompletionSuggestionType {
    Variable = 0,
    Keyword = 1,
    Enum = 2
  }

  /**
   * Creates a new sheet. Will return an error if the provided sheet cannot be parsed sucessfully.
   */
  export interface CreateSheet {
    Sheet: string;
    IsActive?: boolean;
  }

  export interface CreateSheetResponse {
    /**
     * Entity type: Script
     */
    ID: number;
    HasErrors: boolean;
    Errors: ParseSheetError[];
    CompletionSuggestions: CompletionSuggestion[];
    AST: { [key: string]: any };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Deletes a sheet.
   */
  export interface DeleteSheet {
    /**
     * Entity type: Script
     */
    ID: number;
  }

  /**
   * Retrieves the completion options for the provided node that is part of the provided AST.
   * 
   * If no AST is provided this service will return the completion options that a new sheet would have.
   * 
   * The completion options consist of three possible values:
   * 
   * - Alternatives: with which nodes can the requested node be replaced?
   * - NextStatements: if the node is a statement, which statements can come after it?
   * - SelfCompletion: if a node contains child nodes, what are the possible node types for those children?
   */
  export interface GetNodeCompletionOptions {
    AST?: { [key: string]: any };
    NodeID?: number;
  }

  export interface GetNodeCompletionOptionsResponse {
    Result: NodeCompletionResult;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface NodeCompletionResult {
    Alternatives: NodeCompletionResult.ICompletionNode[];
    NextStatements: NodeCompletionResult.ICompletionNode[];
    SelfCompletion: any;
  }

  export namespace NodeCompletionResult {
    export interface ICompletionNode {
      NodeType: EVA.Core.NodeTypes;
    }

  }

  /**
   * Returns the available documentation at the provided position (column/line number) in the sheet.
   */
  export interface GetSheetDocumentation {
    /**
     * The sheet that you would like to have documentation for
     */
    Sheet: string;
    /**
     * The 1-based column number of the position in the sheet that you would like documentation for.
     * Min: 1
     */
    Column: number;
    /**
     * The 1-based line number of the position in the sheet that you would like documentation for.
     * Min: 1
     */
    Line: number;
  }

  export interface GetSheetDocumentationResponse {
    /**
     * The documentation for the requested position. Will be null if there is no available documentation.
     */
    Documentation?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns a list of revisions for a specific sheet.
   */
  export interface ListSheetHistory {
    /**
     * ID of the sheet
     * Entity type: Sheet
     */
    ID: number;
    InitialPageConfig?: EVA.Core.PageTokenConfig<ListSheetHistoryFilter>;
    PageToken?: string;
  }

  export interface ListSheetHistoryFilter {
  }

  export interface ListSheetHistoryResponse {
    Results?: ListSheetHistoryItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListSheetHistoryItem {
    /**
     * Entity type: Script
     */
    ID: number;
    Revision: number;
    Name: string;
    Source: string;
    /**
     * Entity type: User
     */
    UserID: number;
    UserFullName: string;
    Date: string;
    IsActive?: boolean;
  }

  /**
   * Returns a list of sheets, optionally filtered.
   */
  export interface ListSheets {
    InitialPageConfig?: EVA.Core.PageTokenConfig<ListSheetsFilter>;
    PageToken?: string;
  }

  export interface ListSheetsFilter {
    /**
     * Entity type: Sheet
     */
    ID?: number;
    Name: string;
    Source: string;
    IsActive?: boolean;
  }

  export interface ListSheetsResponse {
    Results?: ListSheetItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListSheetItem {
    /**
     * Entity type: Script
     */
    ID: number;
    Name: string;
    Source: string;
    Revision: number;
    /**
     * Entity type: User
     */
    UserID: number;
    UserFullName: string;
    Date: string;
    IsActive?: boolean;
  }

  /**
   * Attempts to parse the provided sheet, returning whether or not it was successful, any errors and possibly autocomplete suggestions.
   */
  export interface ParseSheet {
    Sheet: string;
  }

  export interface ParseSheetResponse {
    HasErrors: boolean;
    Errors: ParseSheetError[];
    CompletionSuggestions: CompletionSuggestion[];
    AST: { [key: string]: any };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Provides the ability to do a what-if to see how the provided OrderID would be fulfilled.
   * 
   * This will use the existing sheets with the ability to specify additional new sheets (by not specifying an ID)
   * or modify an existing sheet (by providing the ID of an existing sheet).
   */
  export interface PreviewOrderFulfillment {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Order?: PreviewOrderFulfillment.OrderModel;
    Stock?: PreviewOrderFulfillment.StockModel;
    CurrentTimeOverride?: string;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitIDs?: number[];
    /**
     * Entity type: Script
     */
    ScriptIDs?: number[];
    /**
     * If you specify sheets they will either be used as an additional sheet to consider when calculating order fulfillment, or as a replacement for an existing sheet based on whether the ID of the sheet is provided or not.
     * 
     * Specifying additional sheets can be used as a 'what-if' tool; what would happen to this order if I change this sheet to do this, etc.
     */
    Sheets?: PreviewOrderFulfillment.SheetToPreview[];
    /**
     * If specified as true the fulfillment preview will base its result on fulfillments that have already been created for the order. For example if the order has already been exported to a certain supplier
     * then the preview service will take that into account and not consider that supplier to be a fulfillment option anymore. If specified as false (the default) the service considers the order to be a clean slate
     * for which all options are still open.
     */
    UseCurrentFulfillments?: boolean;
  }

  export namespace PreviewOrderFulfillment {
    export interface StockModel {
      OverrideIsDelta: boolean;
      StockOverrides: PreviewOrderFulfillment.StockOverride[];
    }

    export interface StockOverride {
      /**
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      QuantityAvailable: number;
    }

    export interface OrderModel {
      /**
       * Entity type: OrganizationUnit
       */
      SoldFromOrganizationUnitID?: number;
      Lines: PreviewOrderFulfillment.OrderLineModel[];
      /**
       * Entity type: Address
       */
      ShippingAddressID?: number;
      /**
       * Entity type: Address
       */
      BillingAddressID?: number;
      /**
       * Entity type: User
       */
      CustomerID?: number;
    }

    export interface OrderLineModel {
      /**
       * Entity type: Product
       */
      ProductID: number;
      UnitPrice?: number;
      QuantityOrdered?: number;
      CommitmentStatus: EVA.Core.OrderLineCommitmentStatus;
      /**
       * Entity type: OrganizationUnit
       */
      FulfillmentOrganizationUnitID?: number;
    }

    export interface SheetToPreview {
      /**
       * Entity type: Script
       */
      ID?: number;
      Source: string;
    }

  }

  export interface PreviewOrderFulfillmentResponse {
    Fulfillments: PreviewOrderFulfillmentResponse.Fulfillment[];
    OrderLines: PreviewOrderFulfillmentResponse.OrderLineResult[];
    ValidSuppliers: PreviewOrderFulfillmentResponse.ValidSupplier[];
    Rejections: PreviewOrderFulfillmentResponse.OrganizationUnitFulfllmentRejection[];
    HasErrors: boolean;
    Errors: ParseSheetError[];
    CompletionSuggestions: CompletionSuggestion[];
    AST: { [key: string]: any };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace PreviewOrderFulfillmentResponse {
    export interface Fulfillment {
      Shipments: PreviewOrderFulfillmentResponse.Shipment[];
      Score: number;
      ScoreExplanation: FulfillmentScoreExplanation;
    }

    export interface Shipment {
      FulfillmentAction: string;
      Sheet: PreviewOrderFulfillmentResponse.Sheet;
      Supplier: PreviewOrderFulfillmentResponse.OrganizationUnit;
      OrderLines: PreviewOrderFulfillmentResponse.OrderLine[];
      Score?: number;
      Data: { [key: string]: any };
      ScoreExplanation: FulfillmentScoreExplanation;
    }

    export interface Sheet {
      /**
       * Entity type: Sheet
       */
      ID?: number;
      Name: string;
    }

    export interface OrganizationUnit {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
      BackendID: string;
    }

    export interface OrderLine {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      Score: number;
      Quantity: number;
      ScoreExplanations: ScoreExplanation[];
    }

    export interface OrderLineResult {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      GroupID?: string;
      PotentialSuppliers: OrderLineResult.PotentialSupplier[];
    }

    export namespace OrderLineResult {
      export interface PotentialSupplier {
        /**
         * Entity type: OrganizationUnit
         */
        OrganizationUnitID: number;
        Score: number;
        Sheet: string;
      }

    }

    export interface ValidSupplier {
      /**
       * Entity type: OrganizationUnit
       */
      SupplierID: number;
      SupplierName: string;
      FulfillmentMethod: string;
      Score: number;
      ScoreExplanations: ValidSupplier.ScoreExplanation[];
      /**
       * Entity type: OrderLine
       */
      FulfillableOrderLineIDs: number[];
    }

    export namespace ValidSupplier {
      export interface ScoreExplanation {
        /**
         * Entity type: OrderLine
         */
        OrderLineID: number;
        Score: number;
        Description: string;
        Expression: string;
        Variables: { [key: string]: any };
      }

    }

    export interface OrganizationUnitFulfllmentRejection {
      /**
       * Entity type: OrganizationUnit
       */
      SupplierOrganizationUnitID?: number;
      SupplierOrganizationUnitName?: string;
      FulfillmentMethods: OrganizationUnitFulfllmentRejection.FulfillmentMethod[];
    }

    export namespace OrganizationUnitFulfllmentRejection {
      export interface FulfillmentMethod {
        Name: string;
        Reasons: OrganizationUnitFulfllmentRejection.RejectionReason[];
      }

      export interface RejectionReason {
        Reason: string;
        Description: string;
        /**
         * Entity type: Product
         */
        ProductIDs: number[];
        /**
         * Entity type: OrderLine
         */
        OrderLineIDs: number[];
      }

    }

  }

  export interface ScoreExplanation {
    Description: string;
    Expression: string;
    PreviousScore: number;
    NewScore: number;
    DeltaScore: number;
    Variables: { [key: string]: any };
  }

  export interface FulfillmentScoreExplanation {
    Description: string;
    ScoreExplanations: ScoreExplanation[];
  }

  /**
   * Updates an existing sheet. Will return an error if the provided sheet cannot be parsed sucessfully.
   */
  export interface UpdateSheet {
    /**
     * Entity type: Sheet
     */
    ID: number;
    Sheet: string;
    IsActive?: boolean;
  }

}
